#
# Copyright (c) 2012 Mellanox Technologies. All rights reserved.
#
# This Software is licensed under one of the following licenses:
#
# 1) under the terms of the "Common Public License 1.0" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/cpl.php.
#
# 2) under the terms of the "The BSD License" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/bsd-license.php.
#
# 3) under the terms of the "GNU General Public License (GPL) Version 2" a
#    copy of which is available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/gpl-license.php.
#
# Licensee has the right to choose one of the above licenses.
#
# Redistributions of source code must retain the above copyright
# notice and one of the license notices.
#
# Redistributions in binary form must reproduce both the above copyright
# notice, one of the license notices in the documentation
# and/or other materials provided with the distribution.
#
#

# KMP is disabled by default
%{!?KMP: %global KMP 0}

%{!?sysctl_update: %global sysctl_update 1}
%{!?MEMTRACK: %global MEMTRACK 0}

%{!?KVERSION: %global KVERSION %(uname -r)}
%global kernel_version %{KVERSION}
%global krelver %(echo -n %{KVERSION} | sed -e 's/-/_/g')
# take path to kernel sources if provided, otherwise look in default location (for non KMP rpms).
%{!?K_SRC: %global K_SRC /lib/modules/%{KVERSION}/build}

%{!?_name: %global _name mlnx-en}
%{!?_version: %global _version 2.2}
%{!?_release: %global _release 1.0.1.0.gdf6fefb}

Name: %{_name}
Group: System Environment
Version: %{_version}
Release: %{_release}%{?_dist}
License: GPLv2 or BSD
Url: http://www.mellanox.com
Group: System Environment/Kernel
Vendor: Mellanox Technologies
Source0: %{_name}-%{_version}.tgz
Provides: %{_name}
%if "%{KMP}" == "1"
Conflicts: mlnx_en
%endif
BuildRoot: %{?build_root:%{build_root}}%{!?build_root:/var/tmp/MLNX_EN}
Summary: mlnx-en kernel module(s)
%description
ConnectX Ehternet device driver
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz

%package KMP
Summary: mlnx-en kernel module(s)
Group: System/Kernel
%description KMP
mlnx-en kernel module(s)
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz

%package doc
Summary: Documentation for the Mellanox Ethernet Driver for Linux
Group: System/Kernel

%description doc
Documentation for the Mellanox Ethernet Driver for Linux
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz

%package sources
Summary: Sources for the Mellanox Ethernet Driver for Linux
Group: System Environment/Libraries

%description sources
Sources for the Mellanox Ethernet Driver for Linux
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz

%package utils
Summary: Utilities for the Mellanox Ethernet Driver for Linux
Group: System Environment/Libraries
conflicts: ofed-scripts

%description utils
Utilities for the Mellanox Ethernet Driver for Linux
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz

# build KMP rpms?
%if "%{KMP}" == "1"
%global kernel_release() $(make -C %{1} kernelrelease | grep -v make)
BuildRequires: %kernel_module_package_buildreqs
%{kernel_module_package}
%else # not KMP
%global kernel_source() %{K_SRC}
%global kernel_release() %{KVERSION}
%global flavors_to_build default
%package -n mlnx_en
Version: %{_version}
Release: %{_release}
Requires: coreutils
Requires: kernel
Requires: pciutils
Requires: grep
Requires: perl
Requires: procps
Requires: module-init-tools
Group: System Environment/Base
Summary: Ethernet NIC Driver
%description -n mlnx_en
ConnectX Ehternet device driver
The driver sources are located at: http://www.mellanox.com/downloads/Drivers/mlnx-en-2.2-1.0.1.tgz
%endif #end if "%{KMP}" == "1"

%if "%{_vendor}" == "suse"
%global install_mod_dir updates
%debug_package
%endif

%if "%{_vendor}" == "redhat"
%if 0%{?fedora}
%global install_mod_dir updates
%else
%global install_mod_dir extra/%{name}
%endif
%global __find_requires %{nil}
%endif

%prep
%setup
set -- *
mkdir source
mv "$@" source/
mkdir obj

%build
rm -rf $RPM_BUILD_ROOT
export EXTRA_CFLAGS='-DVERSION=\"%version\"'
for flavor in %{flavors_to_build}; do
	rm -rf obj/$flavor
	cp -r source obj/$flavor
	cd $PWD/obj/$flavor
	export KSRC=%{kernel_source $flavor}
	export KVERSION=%{kernel_release $KSRC}
	export MLNX_EN_PATCH_PARAMS="--kernel $KVERSION --kernel-sources $KSRC"
	%if "%{MEMTRACK}" == "1"
		export MLNX_EN_PATCH_PARAMS="$MLNX_EN_PATCH_PARAMS --with-memtrack"
	%endif
	./scripts/mlnx_en_patch.sh $MLNX_EN_PATCH_PARAMS
	make KSRC=$KSRC V=0
	cd -
done
gzip -c source/scripts/mlx4_en.7 > mlx4_en.7.gz

cd source/ofed_scripts/utils
python setup.py build
cd -

%install
export INSTALL_MOD_PATH=$RPM_BUILD_ROOT
export INSTALL_MOD_DIR=%install_mod_dir
for flavor in %{flavors_to_build}; do
	cd $PWD/obj/$flavor
	export KSRC=%{kernel_source $flavor}
	export KVERSION=%{kernel_release $KSRC}
	make install KSRC=$KSRC MODULES_DIR=$INSTALL_MOD_DIR DESTDIR=$RPM_BUILD_ROOT KERNELRELEASE=$KVERSION
	# Cleanup unnecessary kernel-generated module dependency files.
	find $INSTALL_MOD_PATH/lib/modules -iname 'modules.*' -exec rm {} \;
	cd -
done

# Set the module(s) to be executable, so that they will be stripped when packaged.
find %{buildroot} -type f -name \*.ko -exec %{__chmod} u+x \{\} \;

%if "%{_vendor}" == "redhat"
%if "%{KMP}" == "1"
%{__install} -d $RPM_BUILD_ROOT%{_sysconfdir}/depmod.d/
for module in `find $RPM_BUILD_ROOT/ -name '*.ko'`
do
ko_name=${module##*/}
mod_name=${ko_name/.ko/}
mod_path=${module/*%{name}}
mod_path=${mod_path/\/${ko_name}}
echo "override ${mod_name} * weak-updates/%{name}${mod_path}" >> $RPM_BUILD_ROOT%{_sysconfdir}/depmod.d/%{name}.conf
done
%endif
%endif

install -D -m 644 mlx4_en.7.gz $RPM_BUILD_ROOT/%{_mandir}/man7/mlx4_en.7.gz

install -D -m 755 source/ofed_scripts/common_irq_affinity.sh $RPM_BUILD_ROOT/%{_sbindir}/common_irq_affinity.sh
install -D -m 755 source/ofed_scripts/set_irq_affinity.sh $RPM_BUILD_ROOT/%{_sbindir}/set_irq_affinity.sh
install -D -m 755 source/ofed_scripts/show_irq_affinity.sh $RPM_BUILD_ROOT/%{_sbindir}/show_irq_affinity.sh
install -D -m 755 source/ofed_scripts/set_irq_affinity_bynode.sh $RPM_BUILD_ROOT/%{_sbindir}/set_irq_affinity_bynode.sh
install -D -m 755 source/ofed_scripts/set_irq_affinity_cpulist.sh $RPM_BUILD_ROOT/%{_sbindir}/set_irq_affinity_cpulist.sh
install -D -m 755 source/ofed_scripts/sysctl_perf_tuning $RPM_BUILD_ROOT/sbin/sysctl_perf_tuning

install -D -m 644 source/scripts/mlx4_en.modprobe.conf $RPM_BUILD_ROOT/etc/modprobe.d/mlx4_en.conf

mkdir -p $RPM_BUILD_ROOT/%{_prefix}/src
cp -r source $RPM_BUILD_ROOT/%{_prefix}/src/%{name}-%{version}

touch ofed-files
cd source/ofed_scripts/utils
python setup.py install -O1 --root=$RPM_BUILD_ROOT --record ../../../ofed-files
cd -

if [[ "$(ls %{buildroot}/%{_bindir}/tc_wrap.py* 2>/dev/null)" != "" ]]; then
	echo '%{_bindir}/tc_wrap.py*' >> ofed-files
fi

%postun doc
if [ -f %{_mandir}/man7/mlx4_en.7.gz ]; then
	exit 0
fi

%if "%{KMP}" != "1"
%post -n mlnx_en
/sbin/depmod -r -ae %{KVERSION}

%postun -n mlnx_en
if [ $1 = 0 ]; then  # 1 : Erase, not upgrade
	/sbin/depmod -r -ae %{KVERSION}
fi
%endif

%post -n mlnx-en-utils
%if "%{sysctl_update}" == "1"

sysctl -q -w net.ipv4.tcp_timestamps=0
sysctl -q -w net.ipv4.tcp_sack=0
sysctl -q -w net.ipv4.tcp_low_latency=1
sysctl -q -w net.core.netdev_max_backlog=250000
sysctl -q -w net.core.rmem_max=16777216
sysctl -q -w net.core.wmem_max=16777216
sysctl -q -w net.core.rmem_default=16777216
sysctl -q -w net.core.wmem_default=16777216
sysctl -q -w net.core.optmem_max=16777216
sysctl -q -w net.ipv4.tcp_rmem="4096 87380 16777216"
sysctl -q -w net.ipv4.tcp_wmem="4096 65536 16777216"

if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf
cat << EOF >> /etc/sysctl.conf
## MLXNET tuning parameters ##
net.ipv4.tcp_timestamps = 0
net.ipv4.tcp_sack = 0
net.ipv4.tcp_low_latency = 1
net.core.netdev_max_backlog = 250000
net.core.rmem_max = 16777216
net.core.wmem_max = 16777216
net.core.rmem_default = 16777216
net.core.wmem_default = 16777216
net.core.optmem_max = 16777216
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
## END MLXNET ##
EOF
fi

%endif

# END of post utils

%postun -n mlnx-en-utils
if [ $1 = 0 ]; then  # 1 : Erase, not upgrade
# Clean sysctl.conf
if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf
fi

fi
# END of postun utils

%clean
rm -rf %{buildroot}

%if "%{KMP}" != "1"
%files -n mlnx_en
/lib/modules/%{KVERSION}/
%endif

%files doc
%defattr(-,root,root,-)
%{_mandir}/man7/mlx4_en.7.gz

%files sources
%defattr(-,root,root,-)
%{_prefix}/src/%{name}-%{version}

%files utils -f ofed-files
%defattr(-,root,root,-)
%{_sbindir}/*
/etc/modprobe.d/*
/sbin/*
%if "%{KMP}" == "1"
%if "%{_vendor}" == "redhat"
%{_sysconfdir}/depmod.d/*
%endif # end rh
%endif

%changelog
* Mon Mar 24 2014 Alaa Hleihel <alaa@mellanox.com>
- Use one source rpm for KMP and none-KMP rpms.
* Tue May 1 2012 Vladimir Sokolovsky <vlad@mellanox.com>
- Created spec file for mlnx_en
